import os
import re
import multiprocessing
from multiprocessing.pool import ThreadPool
import time
from pylint import epylint
from py_gardener.InternalTestBase import InternalTestBase


class TestPylint(InternalTestBase):
    """ Test that we conform to Pylint. """

    def test_pylint(self):
        pool = ThreadPool(multiprocessing.cpu_count())
        files = []
        for file_ in self.list_project_files():
            rc_path = os.path.abspath(file_)
            while not os.path.isfile(os.path.join(rc_path, '.pylintrc')):
                assert rc_path != self.ROOT_DIR and rc_path != os.path.sep
                rc_path = os.path.abspath(rc_path + "/..")
            files.append([file_, rc_path])

        line_length = self.OPTIONS.get('max-line-length', 80)
        processes = [pool.apply_async(
            lambda f: epylint.py_run('%s --rcfile=%s --max-line-length=%s' % (
                f[0],
                os.path.join(f[1], ".pylintrc"),
                line_length
            ), return_std=True),
            [[file_path, rc_file_path]]
        ) for file_path, rc_file_path in files]

        while not all(p.ready() for p in processes):
            time.sleep(0.2)

        result = []
        for process in processes:
            stdout, stderr = map(lambda e: e.getvalue(), process.get())
            if "Your code has been rated at 10.00/10" not in stdout:
                for log in stdout.split("\n"):
                    if re.match(r"^\*{13} Module [a-zA-Z0-9._]+$", log):
                        continue
                    if re.match(r"^\s[-]+$", log):
                        continue
                    if re.match(
                        r"^\sYour code has been rated at \d+\.\d+/10 "
                        r"\(previous run: \d+\.\d+/10, [+\-]\d+\.\d+\)$",
                        log
                    ):
                        continue
                    if re.match(r"^\s*$", log):
                        continue
                    result.append(log.strip())
            if re.match(
                r"^Using config file [/A-Za-z0-9\-_]+?\.pylintrc\n$",
                stderr
            ):
                continue
            if re.match(r"^\s*$", stderr):
                continue
            result.append(stderr.strip())
        pool.close()

        if len(result) > 0:
            for log in result:
                print(log)
            print("PyLint Problems: %s" % len(result))
        assert len(result) == 0



import re
import os
import subprocess
from py_gardener.InternalTestBase import InternalTestBase


class TestVersionConsistent(InternalTestBase):
    """ Test setup.py version doesn't fall behind git tag. """

    def test_version_consistent(self):
        setup_path = os.path.join(self.ROOT_DIR, "setup.py")
        if os.path.isfile(setup_path):
            v1 = subprocess.check_output(
                'git describe --tags --abbrev=0'.split()
            ).strip().decode("utf-8")
            with open(setup_path) as f:
                v2 = re.search("version='([0-9.]+)',", f.read()).groups()[0]
            assert (
                list(map(int, v1.split("."))) <= list(map(int, v2.split(".")))
            )



#!/usr/bin/env python
"""
Py-Gardener
----------------
Enforces best practices in python project.
Links
`````
* `github <https://github.com/loopmediagroup/py-gardener>`_
"""
import os
from setuptools import setup

setup(
    name='py-gardener',
    version='0.6.6',
    url='https://github.com/loopmediagroup/py-gardener',
    license='MIT',
    author='Lukas Siemon',
    author_email='lukas.siemon@getintheloop.ca',
    maintainer='Lukas Siemon',
    maintainer_email='lukas.siemon@getintheloop.ca',
    description='Basic Tests for python project to enforce best practices.',
    long_description=__doc__,
    packages=[x[0] for x in os.walk("py_gardener")],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'pycodestyle>=2.4.0',
        'pylint>=1.8.4',
        'pytest>=3.5.1',
        'pytest-cov>=2.5.1'
    ],
    keywords=['Python', 'Testing', 'Best Practices'],
    classifiers=[]
)
